;TIMEFIX.ASM: A utility with three clock-related functions (resets time by a
; user-selectable number of seconds per day; resets for daylight-saving
; changes; and sets exit code to run other programs only when specified).
;Complete description in accompanying TIMEFIX.DOC.

CSEG	SEGMENT
	ASSUME	CS:CSEG,DS:CSEG

	ORG	0100H

START:
	JMP SHORT	DAY	;Skip over data variables so program doesn't try to
				; execute them as instructions
LASTDAY	DW	?	;Bytes 2-3 in file, where record of last day program
				; was run is kept (changes each day run)
DAYLITE	DB	?	;Byte 4; record of whether daylight saving time was
				; in effect when program was last run
RUNS		DB	0	;Byte 5; record of number of times program run today

TABLE	DW	0,31,59,90,120,151,181,212,243,273,304,334
			;Number of days to add to current month to get day of year

DAY:	CALL	NPARAMS	;Get parameters for nos. of secs. to be added daily
	CALL	GETNAME	;Put this program's path & filename into PFILE variable
	MOV	AH,2AH	;Get DOS date
	INT	21H
	MOV	YEAR,CX	;Put year into YEAR variable for later use
	MOV	MONTH,DH	;Put month into MONTH variable for later use
	MOV	DAYM,DL	;Put day of month into DAYM variable for later use
	CALL	DAYLIGHT	;Do standard/daylight time switch if necessary
	CALL	LEAP		;Adjust for leap year if necessary
	DEC	BL		;Deduct one from table index because first entry is 0
	SHL	BX,1		;Multiply index by 2 because entries are 2 bytes
	ADD	AX,TABLE[BX]	;Add table entry pointed to by index register
	SUB	DH,DH		;Clear month from DX, leaving day
	ADD	AX,DX		;Add day of current month to cumulative days
	MOV	YEARDAY,AX	;Put day of year in memory storage location
	JMP	FILE

LEAP:	MOV	BL,DH		;Put month in index register
	CMP	BL,3		;Has month reached March?
	JB	NLEAP		;If not, leap year doesn't matter in calculating day
	MOV	AL,CL		;If after Feb., divide year by 4
	SUB	AH,AH
	MOV	CL,4
	DIV	CL
	CMP	AH,0		;Is there a nonzero remainder?
	JNE	NLEAP		;If not, not leap year
	MOV	AX,1		;If so, leap year; add 1 to day of year after Feb.
	RET
NLEAP:
	SUB	AX,AX		;If not leap year, don't add 1
	RET

DAYLIGHT:			;Handles standard/daylight time
	CMP	DH,4		;Has April been reached?
	JL	QU1		;If not yet April, exit via standard time code
	JE	MORE		;If April, check which time should be in effect
	CMP	DH,10		;Has October been reached?
	JL	QU2		;If in between, exit via daylight time code
	JMP	NX2		;If Oct. reached, check which time should be in effect
MORE:	PUSH	CX		;Save date parameters
	MOV	BX,1		;Initialize registers to check for first Sunday in April
	MOV	CX,5		; (BX=first date to check; CX=counter for loop below)
M1:	CMP	DL,BL		;Is date April 1?
	JNE	NXT		;If not, check next day
	CMP	AL,BL		;If so, is day of week above Sunday?
	JL	NXT		;If not, check next day
	POP	CX		;Restore date parameters
	JMP	SHORT QU1	;First Sunday not here yet; stay on standard time
NXT:	INC	BX		;If prior tests failed, check through range in which
	LOOP	M1		; first Sunday can be
	POP	CX		;Restore date parameters
	CMP	DL,6		;Is date April 6?
	JNE	QU2		;If not, must be higher; exit via daylight time code
	CMP	AL,6		;If day above Sat., first Sunday must have been reached
	JE	QU1		;If not, first Sun. not yet; exit via standard time code
	;JMP	SHORT QU2	;After 1st Sun. in April; exit via daylight time code

QU2:	PUSH	AX		;Handles operations during daylight saving time
	PUSH	BX
	PUSH	CX
	PUSH	DX
	CALL	OPEN		;Open program's own executable file
	CALL	FIL		;Read file byte that flags standard/daylight time
	CMP	DAYLITE,0	;File byte initialized?
	JNE	NX1		;If initialized (not zero), continue
	CALL	INIT		;If not initialized, print "initializing" message...
	CALL	SUMM		;...and initialize to "D" before quitting
	JMP	SHORT QU
NX1:	CMP	DAYLITE,"S"	;Standard time in effect when program last run?
	JE	NX1A		;If so, proceed to switch to daylight time
	JMP	SHORT QU	;If not, no change needed; quit
NX1A:	MOV	AH,2CH	;Get DOS time
	INT	21H
	CMP	CH,23		;Is day in last hour?
	JNE	AD
	JMP	SHORT QU	;If so, wait to change time until run when it isn't
AD:	CALL	DAYL		;Switch to daylight time by adding an hour to clock...
	CALL	SUMM		;...and toggling flag to "D" before quitting
	JMP	SHORT QU

QU1:	PUSH	AX
	PUSH	BX
	PUSH	CX
	PUSH	DX
	CALL	OPEN		;Open program's own executable file
	CALL	FIL		;Read file byte that flags standard/daylight time
	CMP	DAYLITE,0	;File byte initialized?
	JNE	NX4		;If initialized (not zero), continue
	CALL	INIT		;If not initialized, print "initializing" message...
	CALL	WINT		;...and initialize to "S" before quitting
	JMP	SHORT QU
NX4:	CMP	DAYLITE,"D"	;Daylight time in effect when program last run?
	JE	NX4A		;If so, proceed to switch to standard time
	JMP	SHORT QU	;If not, no change needed; quit
NX4A:	MOV	AH,2CH	;Get DOS time
	INT	21H
	CMP	CH,0		;Is day in first hour?
	JNE	SB		;If not, proceed to change time
	JMP	SHORT QU	;If so, wait until next run after hour changes
SB:	CALL	STAN		;Switch to standard time by subtracting an hour from
	CALL	WINT		; clock...and toggling flag to "S" before quitting.

QU:	POP	DX
	POP	CX
	POP	BX
	POP	AX
	MOV	BX,HANDLE		;Copy file handle no. into BX
	MOV	AH,3EH		;Close file
	INT	21H
	;MOV	AX,4C00H	;[Close file and terminate--uncomment this if only
	;INT	21H		; daylight/standard switching wanted]
	RET

NX2:	CMP	DH,10		;Has October been passed?
	JG	QU1		;If so, exit via standard time code
	CMP	DL,24		;If not, have 24 days passed within October?
	JG	NX2A		;If so, check whether standard time should be in force
	JMP	QU2		;If not, exit via daylight time code

NX2A:	PUSH	CX		;Save date parameters
	MOV	BL,25		;Initialize registers to check for last Sunday in Oct.
	MOV	BH,1		; (BL=first date to check; BH=first day of week to
	MOV	CX,6		; check; CX=counter for loop below)
M2:	CMP	DL,BL		;Is day October 25?
	JNE	NEX		;If not, check next day
	CMP	AL,BH		;If so, is day of week above Sunday?
	JL	NEX		;If below Monday, continue
	POP	CX		;Restore date parameters
	JMP	QU2		;Last Sunday not here yet; quit with daylight time code
NEX:	INC	BL		;If neither true, continue check through range in which
	INC	BH		; last Sunday can be
	LOOP	M2
	POP	CX		;Restore date parameters
	JMP	SHORT QU1	;If last Sunday here, exit via standard time code

DAYL:	CMP	DFLAG,0	;Daylight/standard change not wanted?
	JNE	DNOT		;If so, quit without changing
	ADD	CH,1		;If not last hour of day, add an hour
	MOV	AH,2DH	;Reset time, adding one hour for daylight saving time
	INT	21H

	PUSH	CX
	PUSH	DX
	MOV	DX,OFFSET TEXT1	;Load DX with address of message text string
	CALL	SCRN		;Print message to screen with function 9
	POP	DX
	POP	CX
DNOT:	RET

STAN:	CMP	DFLAG,0	;Daylight/standard change not wanted?
	JNE	SNOT		;If so, quit without changing
	SUB	CH,1		;If not first hour of day, subtract an hour
	MOV	AH,2DH	;Reset time, subtracting one hour for standard time
	INT	21H

	PUSH	CX
	PUSH	DX
	MOV	DX,OFFSET TEXT2	;Load DX with address of message text string
	CALL	SCRN		;Print message to screen with function 9
	POP	DX
	POP	CX
SNOT:	RET

WINT:	MOV	DAYLITE,"S"	;Reset flag byte to show standard time in effect
	MOV	FILFLAG,"W"	;Flag FIL routine to write to file
	CALL	FIL		;Write it to specified byte of program's file
	RET

SUMM:	MOV	DAYLITE,"D"	;Reset flag byte to show daylight time in effect
	MOV	FILFLAG,"W"	;Flag FIL routine to write to file
	CALL	FIL		;Write it to specified byte of program's file
	RET

INIT:	PUSH	AX
	PUSH	DX
	MOV	DX,OFFSET TEXT	;Load DX with address of message text string
	CALL	SCRN		;Print message to screen with function 9
	POP	DX
	POP	AX
	RET

FIL:	PUSH	AX
	PUSH	BX
	PUSH	CX
	PUSH	DX
	MOV	DX,4			;One of two parameters to reset file pointer
	CALL	PREP			;Reset file pointer

	MOV	DX,OFFSET DAYLITE	;Copy address of file data buffer into DX
	MOV	CX,1			;Specify that 1 byte is to be read or written
	CMP	FILFLAG,"W"		;Is a write desired?
	JE	WRIT
	MOV	AH,3FH		;Read file and put data read into buffer
	JMP	SHORT EXE1
WRIT:	MOV	AH,40H		;Write to file from buffer
EXE1:	INT	21H
	POP	DX
	POP	CX
	POP	BX
	POP	AX
	RET

OPEN:	MOV	DX,OFFSET PFILE	;Copy address of path\filename into DX
	MOV	AL,11000010B	;Set file access
	MOV	AH,3DH		;Open file
	INT	21H
	MOV	HANDLE,AX		;Move file handle no. into memory storage
	RET

PREP:	XOR	CX,CX			;One of two parameters to reset file pointer
	MOV	BX,HANDLE		;Copy file handle no. into BX
	XOR	AL,AL			;Prepare to reposition file read/write pointer
	MOV	AH,42H		;Move file pointer
	INT	21H
	RET

FILE:	CALL	OPEN			;Open program's own executable file
	MOV	DX,2			;One of two parameters to reset file pointer
	CALL	PREP			;Reset file pointer
	MOV	DX,OFFSET LASTDAY	;Copy address of file data buffer into DX
	MOV	CX,2			;Specify that 2 bytes are to be read from file
	MOV	AH,3FH		;Read file and put data read into buffer
	INT	21H

	MOV	AX,LASTDAY		;Copy last day program was run into AX
	MOV	BX,YEARDAY		;Copy current day into BX
	CMP	AX,0			;Has program been run since it was assembled?
	JNE	ON			;If so, proceed to test whether reset necessary
	JMP	SHORT INI		;If not, initialize file without doing reset
ON:	CMP	AX,BX			;Was program last run on same day?
	JNE	ON1			;If not, continue
	JMP	ALT			;If so, close file and quit
ON1:	CMP	AX,365		;Was last run on last day of year?
	JNE	NSET			;If not, no reset needed
	SUB	AX,AX			;If so, reset to zero
NSET:	SUB	BX,AX			;How many days have passed since program was run?
	MOV	DAYS,BX		;Save the number of days in memory
	MOV	DAYS1,BX		;Save another copy
INI:	MOV	AX,YEARDAY		;Update file contents to current day
	MOV	LASTDAY,AX
	MOV	DX,2			;One of two parameters to reset file pointer
	CALL	PREP			;Reset file pointer
	MOV	DX,OFFSET LASTDAY	;Copy address of file data buffer into DX
	MOV	CX,2			;Specify that 2 bytes are to be written to file
	MOV	AH,40H		;Write to file from buffer
	INT	21H

	CMP	DAYS,0		;If not at least one day since program was run,
	JNE	CONT
	JMP	ALT			; quit without changing time
CONT:	CALL	EVOD			;Get seconds to be added based on even & odd days
	MOV	AX,TOTAL		;Move seconds to be added into AX
	CMP	TOTAL,60		;More than 60 seconds to be added?
	JLE	DOWN			;If not, add without recalculating
	SUB	DX,DX
	MOV	CX,60			;If so, convert multiples of 60 to minutes
	IDIV	CX			;Divide seconds to be added (in AX) by 60
	MOV	ADDMINS,AL		;Store minutes
	MOV	ADDSECS,DL		;Store seconds
	JMP	SHORT DWN1		;Proceed to add to time
DOWN:	MOV	ADDSECS,AL		;If no minutes, store seconds straight from AX

DWN1:	MOV	AH,2CH		;Get DOS time
	INT	21H
	MOV	HOURMIN,CX		;Store hour, minute, and
	MOV	SECOND,DH		; second

	CMP	NEGP,0		;Were parameters negative?
	JNE	NEGT			;If so, prepare to subtract time

	CMP	DAYS1,365		;Has date been set backward?
	JAE	NP1			;If so, subtract time despite positive parameters
NP:	ADD	DH,ADDSECS		;Add calculated seconds to proper register
	CMP	DH,59			;Seconds above acceptable range for reset?
	JLE	TST1			;If not, handle minutes
	SUB	DH,60			;If so, subtract 60 seconds and
	ADD	CL,1			; add them as a minute
TST1:	ADD	CL,ADDMINS		;Add calculated minutes to proper register
	CMP	CL,59			;Minutes above acceptable range for reset?
	JLE	TST2			;If not, handle hours
	SUB	CL,60			;If so, subtract 60 minutes and
	ADD	CH,1			; add them as an hour
TST2:	CMP	CH,23			;Hours above acceptable range for reset?
	JG	NTOT			;If so, don't reset
	JMP	SHORT	PTIM		;If not, do positive reset

NEGT:	CMP	DAYS1,365		;Was date set backward?
	JAE	NP			;If so, add time despite negative parameters
NP1:	SUB	DH,ADDSECS		;Subtract calculated seconds
	TEST	DH,80H		;Result negative?
	JZ	NEG2			;If not, handle minutes
	ADD	DH,60			;If so, add 60 seconds and
	SUB	CL,1			; subtract the equivalent minute
NEG2:	SUB	CL,ADDMINS		;Subtract calculated minutes from proper register
	TEST	CL,80H		;Result negative?
	JZ	NEG3			;If not, handle hours
	ADD	CL,60			;If so, add 60 minutes and
	SUB	CH,1			; subtract the equivalent hour
NEG3:	TEST	CH,80H		;Result negative (reset would cross midnight)?
	JNZ	NTOT			;If so, don't reset
	JMP	NTIM			;If not, do negative reset
NTOT:	MOV	TOTAL,0		;Zero seconds to reset
	JMP SHORT	S		;Skip over time reset code

PTIM:	MOV	AH,2DH		;Reset time, adding specified seconds per day
	INT	21H
S:	CMP	PFLAG,0		;Was "S" switch found?
	JE	ON2			;If not, display message before exiting
	JMP	SKIP			;If so, display no message, but exit with
ON2:	PUSH	CX			; appropriate exit code
	PUSH	DX
	CMP	TOTAL,10		;Adding 10 or more seconds?
	JB	BOX1			;If not, leave message box original size
	CMP	TOTAL,100		;Adding 10 to 99 seconds?
	JB	BOX2			;If so, lengthen message box one unit
	CMP	TOTAL,1000		;Adding 100 to 999 seconds?
	JB	BOX3			;If so, lengthen message box two units
	MOV	[TEXT3+43],""	;If 1000 or more, add box characters
	MOV	[TEXT3+44],""	; three
	MOV	[TEXT3+45],""	; units
	MOV	[TEXT3+46],""	; right
	JMP	SHORT BOX1
BOX3:	MOV	[TEXT3+43],""	;If 100 or more, add box characters
	MOV	[TEXT3+44],""	; two units
	MOV	[TEXT3+45],""	; right
	JMP	SHORT BOX1
BOX2:	MOV	[TEXT3+43],""	;If 10-99, add box characters
	MOV	[TEXT3+44],""	; one unit right
BOX1:	MOV	DX,OFFSET TEXT3	;Load DX with address of message text string
	CALL	SCRN			;Print message to screen with function 9
	MOV	ATTRIB,10		;Set intense green attribute for number of seconds
	MOV	AX,TOTAL		;Get total number of seconds to be added
	CALL	PUTNMB		;Embed total seconds in message
	CMP	TOTAL,10		;Adding 10 or more seconds?
	JB	BX1			;If not, leave message box original size
	CMP	TOTAL,100		;Adding 10 to 99 seconds?
	JB	BX2			;If so, lengthen message box one unit
	CMP	TOTAL,1000		;Adding 100 to 999 seconds?
	JB	BX3			;If so, lengthen message box two units
	MOV	[TEXT4+63],""	;If 1000 or more, extend box characters
	MOV	[TEXT4+64],""	; three
	MOV	[TEXT4+65],""	; units
	MOV	[TEXT4+66],""	; right
	JMP	SHORT BX1
BX3:	MOV	[TEXT4+63],""	;If 100 or more, extend box characters
	MOV	[TEXT4+64],""	; two units
	MOV	[TEXT4+65],""	; right
	JMP	SHORT BX1
BX2:	MOV	[TEXT4+63],""	;If 10-99, extend box characters
	MOV	[TEXT4+64],""	; one unit right
BX1:	MOV	DX,OFFSET TEXT4	;Load DX with address of message text string
	CALL	SCRN			;Print message to screen with function 9
	CALL	DTIM			;Display original and reset time
	POP	DX
	POP	CX
	JMP	SKIP

NTIM:	MOV	AH,2DH		;Reset time, adding specified seconds per day
	INT	21H
	CMP	PFLAG,0		;Was "S" switch found?
	JE	ON3			;If not, display message before exiting
	JMP	SKIP			;If so, display no message, but exit with
ON3:	PUSH	CX			; appropriate exit code
	PUSH	DX
	CMP	TOTAL,10		;Adding 10 or more seconds?
	JB	BOX5			;If not, leave message box original size
	CMP	TOTAL,100		;Adding 10 to 99 seconds?
	JB	BOX6			;If so, lengthen message box one unit
	CMP	TOTAL,1000		;Adding 100 to 999 seconds?
	JB	BOX7			;If so, lengthen message box two units
	MOV	[TEXT5+53],""	;If 1000 or more, add box characters
	MOV	[TEXT5+54],""	; three
	MOV	[TEXT5+55],""	; units
	MOV	[TEXT5+56],""	; right
	JMP	SHORT BOX5
BOX7:	MOV	[TEXT5+53],""	;If 100-999, add box characters
	MOV	[TEXT5+54],""	; two units
	MOV	[TEXT5+55],""	; right
	JMP	SHORT BOX5
BOX6:	MOV	[TEXT5+53],""	;If 10-99, add box characters
	MOV	[TEXT5+54],""	; one unit right
BOX5:	MOV	DX,OFFSET TEXT5	;Load DX with address of message text string
	CALL	SCRN			;Print message to screen with function 9
	MOV	ATTRIB,14		;Set intense yellow attribute for number of seconds
	MOV	AX,TOTAL		;Get total number of seconds to be added
	CALL	PUTNMB		;Embed total seconds in message
	CMP	TOTAL,10		;Adding 10 or more seconds?
	JB	BX5			;If not, leave message box original size
	CMP	TOTAL,100		;Adding 10 to 99 seconds?
	JB	BX6			;If so, lengthen message box one unit
	CMP	TOTAL,1000		;Adding 100 to 999 seconds?
	JB	BX7			;If so, lengthen message box two units
	MOV	[TEXT6+75],""	;If 1000 or more, extend box characters
	MOV	[TEXT6+76],""	; three
	MOV	[TEXT6+77],""	; units
	MOV	[TEXT6+78],""	; right
	JMP	SHORT BX5
BX7:	MOV	[TEXT6+75],""	;If 100 or more, extend box characters
	MOV	[TEXT6+76],""	; two units
	MOV	[TEXT6+77],""	; right
	JMP	SHORT BX5
BX6:	MOV	[TEXt6+75],""	;If 10-99, extend box characters
	MOV	[TEXT6+76],""	; one unit right
BX5:	MOV	DX,OFFSET TEXT6	;Load DX with address of message text string
	CALL	SCRN			;Print message to screen with function 9
	CALL	DTIM			;Display original and reset time
	POP	DX
	POP	CX

SKIP:	MOV	RUNS,0		;Day's first run; reset program's run counter
	JMP	SHORT SKP1		;Skip copying run counter from file

ALT:	MOV	DX,5			;One of two parameters to reset file pointer
	CALL	PREP			;Reset file pointer
	MOV	DX,OFFSET RUNS	;Copy address of file data buffer into DX
	MOV	CX,1			;Specify that 1 byte is to be read from file
	MOV	AH,3FH		;Read file and put data read into buffer
	INT	21H

SKP1:	INC	RUNS
	CMP	PFLAG,0		;Was "S" switch found?
	JNE	SKP2			;If so, display no message
	CALL	ALTMSG		;Output day of year & runs today to screen

SKP2:	MOV	DX,5			;One of two parameters to reset file pointer
	CALL	PREP			;Reset file pointer
	MOV	DX,OFFSET RUNS	;Copy address of file data buffer into DX
	MOV	CX,1			;Specify that 1 byte is to be written to file
	MOV	AH,40H		;Write to file from buffer
	INT	21H

	MOV	AL,RUNS		;Quit with exit code equal to times run today
	MOV	AH,4CH		;Select terminate-and-close-file function
	INT	21H

EVOD:	MOV	AX,YEARDAY		;Check whether day of year is even or odd
	SUB	AH,AH
	MOV	BL,2			;Divide last two digits of day of year by 2
	DIV	BL
	CMP	AH,0			;Is there a nonzero remainder?
	JE	EVN			;If not, day of year is even; if so, it's odd
ODD:	MOV	AL,ODDSECS		;Move seconds to add on an odd day of year
	CALL	CNT			;Has more than a day passed since last run?
	JNE	EVN			;If so, add seconds for the last even day missed
	JMP	SHORT Q3		;If all days counted down, exit
EVN:	MOV	AL,EVENSECS		;Move seconds to add on an even day of year
	CALL	CNT			;Has more than a day passed since last run?
	JNE	ODD			;If so, add seconds for the last odd day missed
Q3:	RET				;If all days counted down, exit

CNT:	SUB	AH,AH			;Clear high part of AX
	ADD	TOTAL,AX		;Add seconds
	CMP	DAYS,365		;More than a year of days since program last run?
	JAE	NEG1			;If so, date was set backward; subtract seconds
	DEC	DAYS			;Deduct one from total days since program last run
	JMP	SHORT CMP1
NEG1:	INC	DAYS			;Add one to total days since program run
CMP1:	CMP	DAYS,0		;Was it more than one day?
	RET				;Return to caller to evaluate

ALTMSG:
	PUSH	CX
	PUSH	DX
	MOV	DX,OFFSET TEXTA	;Load DX with address of message text string
	CALL	SCRN		;Print message to screen with function 9
	MOV	ATTRIB,0EH	;Set intense yellow attribute for day of year
	MOV	AX,YEARDAY	;Get day-of-year number
	CALL	PUTNMB	;Embed day of year in message
	MOV	DX,OFFSET TEXTB	;Load DX with address of message text string
	CALL	SCRN		;Print message to screen with function 9
	MOV	ATTRIB,0CH	;Set intense red attribute for year
	MOV	AX,YEAR	;Get year
	CALL	PUTNMB	;Embed year in message

	CMP	RUNS,1	;First run of day?
	JE	T3		;If so, skip displaying time in this text block
	MOV	DX,OFFSET TEXTF	;Point to locations for time display
	CALL	SCRN		;Print message to screen with function 9
	MOV	ATTRIB,11	;Set display attribute
	CALL	T2		;Display time on screen

T3:	MOV	CX,12		;Initialize counter to match month to screen message
	MOV	DX,OFFSET TEXTC	;Point to December text
DATE:	CMP	MONTH,CL	;Does month agree with counter?
	JE	DAT1		;If so, output month and day to screen
	ADD	DX,27		;If not, point to next month's text
	LOOP	DATE		; and try again with next lower month until found
DAT1:	CALL	SCRN		;Print message to screen with function 9
	MOV	ATTRIB,0DH	;Set intense purple attribute for month
	XOR	AH,AH		;Clear high part of AX
	MOV	AL,DAYM	;Get day of month
	CALL	PUTNMB	;Embed day of month in message

	MOV	DX,OFFSET TEXTD	;Load DX with address of message text string
	CALL	SCRN		;Print message to screen with function 9
	MOV	ATTRIB,0AH	;Set intense green attribute for number
	XOR	AH,AH		;Clear high part of AX
	MOV	AL,RUNS	;Get number of runs today
	CALL	PUTNMB	;Embed number in message
	MOV	DX,OFFSET TEXTE	;Load DX with address of message text string
	CALL	SCRN		;Print message to screen with function 9
	POP	DX
	POP	CX
	RET

DTIM:	MOV	DX,OFFSET TEXT7	;Load DX with address of message text string
	CALL	SCRN		;Print message to screen with function 9
	MOV	CX,HOURMIN	;Restore time before reset
	XOR	DL,DL
	MOV	DH,SECOND
	MOV	ATTRIB,12	;Set display attribute
	CALL	TDIS		;Display time before reset
	CMP	TOTAL,0	;Was time actually reset?
	JNE	T1
	MOV	DX,OFFSET TEXT9	;Load DX with address of message text string
	CALL	SCRN		;Print message to screen with function 9
	RET
T1:	MOV	DX,OFFSET TEXT8	;Load DX with address of message text string
	CALL	SCRN		;Print message to screen with function 9
	MOV	ATTRIB,14	;Set display attribute
T2:	MOV	AH,2CH	;Get current DOS time
	INT	21H
	CALL	TDIS		;Display time after reset
	RET

PUNC		MACRO CHAR		;To simplify putting punctuation in displays
		MOV	AL,CHAR	;Output punctuation character
		CALL	KHAND
		ENDM

TDIS:	XOR	AH,AH
	MOV	AL,CH		;Load AL with hour value
	CMP	AL,12		;Is the hour 12? 
	JL	AM		;If less, handle morning case
	MOV	BH,"P" 	;Put ASCII code for P in BH
	JE	OVER		;If hour 12, skip resetting to 12-hour mode
	SUB	AL,12		;Reset by subtracting 12H from higher hour
	JMP	SHORT	OVER	;Proceed to output characters
AM:	MOV	BH,"A"	;Put ASCII code for A in BH
	CMP	AL,0		;Is the hour zero?
	JNE	OVER		;If not, don't reset
	ADD	AL,12		;Reset to 12
OVER:	CMP	AL,10		;Is it a double-digit hour?
	JGE	PUT1		;If not, precede with leading space
	PUSH	AX		;Save hour value
	PUNC	" "		;Output space
	POP	AX		;Restore hour value
PUT1:	CALL	PUTNMB	;Output hour value to decimal digits
COLN:	PUNC	":"		;Output colon
	MOV	AL,CL 	;Load AL with minute value
	CMP	AL,10		;Is it a double-digit minute?
	JGE	PUT2		;If not, precede with leading zero
	PUSH	AX		;Save hour value
	PUNC	"0"		;Output zero
	POP	AX		;Restore hour value
PUT2:	CALL	PUTNMB	;Output minute value as decimal digits
	PUNC	":"		;Output second colon
	MOV	AL,DH 	;Load AL with second value
	CMP	AL,10		;Is it a double-digit second?
	JGE	PUT3		;If not, precede with leading zero
	PUSH	AX		;Save hour value
	PUNC	"0"		;Output zero
	POP	AX		;Restore hour value
PUT3:	CALL	PUTNMB	;Output second value as decimal digits
	PUNC	" "		;Output space
	MOV	AL,BH 	;Output either A or P after space
	CALL	KHAND
	RET

HANDLE	DW	0	;Storage for file handle
YEAR		DW	0	;Storage for year
MONTH		DB	0	;Storage for month
HOURMIN	DW	0	;Storage for hour and minute before reset
SECOND	DB	0	;Storage for second before reset
DAYM		DB	0	;Storage for day of month
YEARDAY	DW	0	;Storage for day of year
DAYS		DW	0	;Storage for number of days since program last ran
DAYS1		DW	0	;Storage for another copy of above
EVENSECS	DB	0	;Number of seconds to add per day (even days)
ODDSECS	DB	0	;Number of seconds to add per day (odd days)
ADDSECS	DB	0	;Storage for seconds to reset time
ADDMINS	DB	0	;Storage for minutes to reset time
TOTAL		DW	0	;Storage for total seconds added in a given run
ATTRIB	DB	0	;Temporary register used in screen output
FILFLAG	DB	0	;To flag whether the FIL routine is to read or write
PFLAG		DB	0	;To flag whether parameters were entered
NEGP		DB	0	;To flag whether parameters are negative
DFLAG		DB	0	;To flag whether to suppress daylight-saving changes

TEXT	DB	13,10,9,"     INITIALIZING STANDARD/DAYLIGHT TIME & DAY-OF-YEAR FLAGS"
	DB	13,10,9,"        To update time, run TIMEFIX after date changes: "
	DB	13,10,9,"              Syntax: TIMEFIX /[S] [D] /[-]n1/n2        "
	DB	13,10,9,"         S = optional, to suppress screen output        "	
	DB	13,10,9,"         D = optional, to stop daylight-saving changes  "	
	DB	13,10,9,"        n1 = seconds, even days; n2 = seconds, odd days ",13,10,10,7,"$"
TEXT1	DB	13,10,9,9,"Ĵ Switching from STANDARD to DAYLIGHT SAVING time ",13,10,7,"$"
TEXT2	DB	13,10,9,9,"Ĵ Switching from DAYLIGHT SAVING to STANDARD time ",13,10,7,"$"

;Any changes to TEXT3-TEXT6 require updated references in PTIM and NTIM code
TEXT3 DB	9,9,"      ͻ    "
	DB	13,10,9,9,"       TIMEFIX adding ","$"
TEXT4	DB	" seconds to time ",13,10,9,9,"      ͼ    ",13,10,"$"

TEXT5 DB	9,"          ͻ    "
	DB	13,10,9,"           TIMEFIX subtracting ","$"
TEXT6	DB	" seconds from time ",13,10,9,"          ͼ    ",13,10,"$"

TEXT7	DB	9,9,"          Time before reset: ","$"
TEXT8	DB	13,10,9,9,"           Time after reset: ","$"
TEXT9	DB	13,10,9,9,"                 Time not reset","$"

TEXTA	DB	13,10,9,9,9,""
	DB	13,10,9,9,9,"     TIMEFIX: Day ","$"
TEXTB	DB	" of ","$"
TEXTC	DB	13,10,9,9,9,"            December ","$"	;Some months padded with
	DB	13,10,9,9,9,"            November ","$"	; null bytes so lines
	DB	13,10,9,9,9,"            October ","$",0	; are of equal length
	DB	13,10,9,9,9,"           September ","$"	; for loop operation
	DB	13,10,9,9,9,"             August ","$",0	; that selects the
	DB	13,10,9,9,9,"              July ","$",0,0	; offset of each string
	DB	13,10,9,9,9,"              June ","$",0,0
	DB	13,10,9,9,9,"              May ","$",0,0,0
	DB	13,10,9,9,9,"             April ","$",0,0
	DB	13,10,9,9,9,"             March ","$",0,0
	DB	13,10,9,9,9,"           February ","$",0
	DB	13,10,9,9,9,"            January ","$",0
TEXTD	DB	13,10,9,9,9,"     This run: number ","$"
TEXTE	DB	" today",13,10,9,9,9,"",13,10,"$"
TEXTF	DB	13,10,9,9,9,"           ","$"
MSG	DB	13,10,"Usage: TIMEFIX /[S] [D] /[-]n1/n2, where n1 is seconds (up to 255)"
		DB	13,10,"  to add or subtract each even day of the year, and n2 each odd day."
		DB	13,10," S is an optional switch to suppress screen output."
		DB	13,10," D is an optional switch to prevent daylight-saving changes.",13,10,"$"

PFILE		DB	80 dup (0)	

;The following procedure converts hex number values to decimal digits
; (supplied to me by Larry Fish of Denver, CO)
;OUTPUT THE INTEGER IN AX

REG1		DW	0		;TEMPORARY REGISTER
SUPRES	DB	0		;TO SUPPRESS LEADING ZEROS

PUTNMB	PROC	NEAR
		MOV	CS:SUPRES,0	;FLAG LEADING ZEROS
		MOV	CS:REG1,AX	;SAVE WORD
		TEST	AX,8000H	;NEGATIVE?
		JZ	INTOT1	;SKIP IF NOT
		NEG	CS:REG1	;NEGATE THE WORD
		PUSH	AX
		MOV	AL,'-'	;PRINT MINUS
		CALL	KHAND		;OUTPUT CHAR
		POP	AX

;Convert from binary to decimal by repeatedly dividing by
;powers of 10 and using the remainder

INTOT1:	PUSH	DX
		MOV	SI,6		;SET POWER POINTER
INTOT2:	MOV	AX,CS:REG1	;GET THE WORD
		XOR	DX,DX
		DIV	CS:POWER[SI]	;DIVIDE BY POWER OF 10
		MOV	CS:REG1,DX	;SAVE REMAINDER
		TEST	AX,0FFFFH	;ZERO DIGIT?
		JNZ	INTOT4	;SKIP IF NOT
		TEST	CS:SUPRES,0FFH ;STILL SUPRESSING?
		JZ	INTOT3	;SKIP IF YES
INTOT4:	INC	CS:SUPRES	;FLAG NO MORE SUPRESS
		OR	AL,30H	;CONVERT DIGIT TO ASCII
		PUSH	SI		;SAVE INDEX
		CALL	KHAND		;PRINT DIGIT
		POP	SI		;RESTORE INDEX
INTOT3:	SUB	SI,2		;POINT TO NEXT POWER
		JNS	INTOT2	;LOOP TILL DONE
		MOV	AL,BYTE PTR CS:REG1	;GET ONES DIGIT
		OR	AL,30H	;CONVERT TO ASCII
		CALL	KHAND		;PRINT IT
		POP	DX
		RET

PUTNMB	ENDP

;POWER OF TEN TABLE

POWER 	DW	10
		DW	100
		DW	1000
		DW	10000

KHAND:PUSH	DX		;Writes character & attribute to screen
	PUSH	CX		;Save codes in calling routine
	PUSH	BX
	PUSH	AX
	PUSH	AX		;Save code to be output
	PUSH	DS		;Save data segment address
	MOV	AX,40H	;Get active display page no. into BH
	MOV	DS,AX
	MOV	BH,DS:BYTE PTR[62H]	;No. from BIOS data segment memory
	POP	DS
	POP	AX		;Restore code to be output
	MOV	BL,ATTRIB	;Load BL with attribute value
	MOV	CX,1		;Load CX with number of times to write character
	MOV	AH,9		;Select BIOS video write-char.-&-attrib. function
	INT	10H		;Write character w/selected attributes
	MOV	AH,3		;Select BIOS read-cursor-position function
	INT	10H		;Read it
	MOV	AH,2		;Select BIOS set-cursor-position function
	ADD	DL,1		;Prepare to move cursor one column right
	INT	10H	 	;Move it
	POP	AX
	POP	BX
	POP	CX
	POP	DX
	RET

NPARAMS:			;Get parameters
	CLD			;Clear the direction flag
	MOV	SI,81H	;Point SI to first parameter character
	CALL	WGET		;Get non-numeric parameter, if any
	CALL	NUMGET	;Process first parameter into numeric value
	CALL	ADP1		;Store first parameter value and continue
	CALL	NUMGET	;Process second parameter into numeric value
	JMP	ADP2		;Store second parameter value and exit

WGET: LODSB			;Get first command-tail character
	CMP	AL,20H	;Is it a space?
	JE	WGET		;If so, loop back to check next character
	CMP	AL,9		;Is it a tab?
	JE	WGET		;If so, loop back to check next character
	CMP	AL,"/"	;Is it a slash?
	JE	WGET		;If so, loop back to check next character
	CMP	AL,","	;Is it a comma?
	JE	WGET		;If so, loop back to check next character
	CMP	AL,"?"	;Is it a question mark?
	JE	MSG1		;Output usage message and quit
	CMP	AL,13		;Is this the end of the line?
	JE	DE2 		;If so, no parameters; continue
	AND	AL,5FH	;Capitalize character if lowercase
	CMP	AL,"S"	;Is it "S"?
	JNE	DE1		;If not, go on
	INC	PFLAG		;Flag that no screen output wanted
	JMP	SHORT WGET	;Check next character
DE1:	CMP	AL,"D"	;Is it "D"?
	JNE	DE2		;If not, go on
	NOT	DFLAG		;Flag that no daylight-saving changes wanted
	JMP	SHORT WGET	;Check next character
DE2:	DEC	SI		;Back up pointer to check for numeric parameters
QU4:	RET

;Get numeric parameter and convert to hex value
NUMGET:	JMP	SHORT GETNUM	;Skip over data storage bytes

SIGN		DB	0		;Flag for handling negative numbers
NUMFLAG	DB	0		;Flag to record that a number's been found
TENBYT	DW	10		;Multiplier for higher-place digits

GETNUM:	LODSB			;Get first char. of command tail
		XOR	CX,CX		;Clear CX for following operations
		MOV	SIGN,0	;Clear sign flag
		MOV	NUMFLAG,0	;Clear number-found flag
		CMP	AL,"-"	;Is parameter preceded by minus?
		JNE	INTIN2	;If not, skip negative-handling code
		NOT	SIGN		;Invert sign flag
INTIN1:	PUSH	CX		;Save accumulated value, if any yet
		LODSB			;Get past minus sign
		POP	CX		;Restore accumulated value
INTIN2:	CMP	AL,13		;Carriage return reached?
		JE	INTIN5	;If so, exit
		SUB	AL,"0"	;See if char.'s a number & convert to hex value
		JL	INTIN4	;If result negative, not number; try again
		CMP	AL,10		;If result 10 or more, not number;
		JGE	INTIN4	; try for a no. again
		INC	NUMFLAG	;Flag that a number's been found
		SUB	AH,AH		;Clear high part of AX
		XCHG	AX,CX		;Switch no. previously found (if any) with new one
		MUL	TENBYT	;Multiply previous no. by 10
		ADD	CX,AX		;Add result in AX to new no. that's now in CX
		JMP	SHORT INTIN1	;Try for another number
INTIN4:	TEST	NUMFLAG,0FFH	;Has any number been found?
		JZ	GETNUM	;If not, start over
INTIN5:	TEST	SIGN,0FFH	;Is number negative?
		JZ	INTIN3	;If not, leave alone
		NEG	CX		;If so, change to corresponding negative value
INTIN3:	MOV	AX,CX		;Store final result in AX
		DEC	SI	;Put back inside CR in case non.-num param. sought
		RET

MSG1:	MOV	DX,OFFSET MSG
	CALL	SCRN		;Print message to screen
	MOV	AX,4C00H	;Terminate
	INT	21H

SCRN:	MOV	AH,9		;Print message
	INT	21H
	RET

ADP1:	CMP	AX,255	;More than 1 byte in AX?
	JG	QU3		;If so, too high a parameter entered; error
	TEST	AX,8000H	;Negative?
	JZ	AD1		;Skip if not
	NEG	AX		;Negate the word
	ADD	NEGP,1	;Set flag for negative parameter
AD1:	ADD	EVENSECS,AL	;Store parameter value
	RET
QU3:	JMP	SHORT MSG1

ADP2:	CMP	AX,255	;More than 1 byte in AX?
	JG	QU3		;If so, too high a parameter entered; error
	TEST	AX,8000H	;Negative?
	JZ	AD2		;Skip if not
	NEG	AX		;Negate the word
	ADD	NEGP,1	;Set flag for negative parameter
AD2:	ADD	ODDSECS,AL	;Store second parameter value
	RET

GETNAME:	MOV	AX,CS		;Get the name of and path
		MOV	ES,AX		; to the executable file and
		MOV	DI,OFFSET PFILE	;store them in PFILE
		CALL	GETPROGNAME	; so the program can open its own file
		RET

;****************************************************************************
; GETPROGNAME copies the name of and path to the executable file that
; corresponds to this program from the program's environment block. On
; entry, DS holds PSP segment address and ES:DI points to the buffer
; that receives the program name.
; (From Jeff Prosise, PC Magazine DOS 6 Techniques & Utilities, 1994.)
;****************************************************************************

getprogname		proc	near
		push	ds		;Save DS
		assume	ds:nothing
		mov	ds,ds:[2Ch]	;DS -> Environment block
		xor	si,si		;Zero SI

gpg1:		lodsb			;Get a character
		or	al,al		;Loop if not zero
		jne	gpg1
		lodsb			;Get the next character
		or	al,al		;Loop if not zero
		jne	gpg1
		add	si,2		;SI -> Program name

gpg2:		lodsb			;Read a character
		cmp	al,"a"	;Capitalize it if it's
		jb	gpg3		; lowercase
		cmp	al,"z"
		ja	gpg3
		and	al,0DFh
gpg3:		stosb			;Copy the byte to the buffer
		or	al,al		;Was the last byte a zero?
		jnz	gpg2		;Loop back if not

		pop	ds		;Restore DS
		assume	ds:cseg
		ret			;Return to caller
getprogname		endp

CSEG	ENDS
	END	START
